@@ -3,13 +3,13 @@ from __future__ import division, print_function
 
 import abc
 import collections
+import copy
 import numpy as np
 
 from pprint import pprint, pformat
-
 from pymatgen.util.string_utils import str_aligned, str_delimited, is_string, list_strings
-from .abiobjects import SpinMode, Smearing, Electrons
-from .pseudos import PseudoTable
+from pymatgen.io.abinitio.abiobjects import SpinMode, Smearing, Electrons
+from pymatgen.io.abinitio.pseudos import PseudoTable
 
 __author__ = "Matteo Giantomassi"
 __copyright__ = "Copyright 2013, The Materials Project"
@@ -17,67 +17,35 @@ __version__ = "0.1"
 __maintainer__ = "Matteo Giantomassi"
 __email__ = "gmatteo at gmail.com"
 
-##########################################################################################
 
-# TODO
-#class ScfStrategyFactory(object):
-#    """
-#    Abstract factory for strategies. Returns a concrete factory
-#
-#    Example: abstract_factory = ScfStrategyFactory()
-#        factory = abstract_factory()
-#        scf_strategy = factory.make_strategy()
-#    """
-#    __metaclass__ = abc.ABCMeta
-#        
-#    #def __init__(self, mode, accuracy):
-#        # Dict with Ground-State strategy subclasses
-#        gs_subclasses = {}
-#        for klass in Scftrategy.__subclasses__():
-#            gs_subclasses[klass.__name__] = klass
-
-
-def select_pseudos(pseudos, structure):
+def select_pseudos(pseudos, structure, ret_table=True):
     """
     Given a list of pseudos and a pymatgen structure, extract the pseudopotentials 
     for the calculation (useful when we receive an entire periodic table).
 
-    Raises ValueError if no pseudo or multiple occurrences are found.
+    Raises:
+        ValueError if no pseudo is found or multiple occurrences are found.
     """
     table = PseudoTable.astable(pseudos)
 
     pseudos = []
     for typ in structure.types_of_specie:
-        # Get list of pseudopotentials in table from atom symbol.
+        # Get the list of pseudopotentials in table from atom symbol.
         pseudos_for_type = table.pseudos_with_symbol(typ)
                                                                              
         if pseudos_for_type is None or len(pseudos_for_type) != 1:
             raise ValueError("Cannot find unique pseudo for type %s" % typ)
                                                                              
         pseudos.append(pseudos_for_type[0])
-                                                                                 
-    return PseudoTable(pseudos)
+
+    if ret_table:
+        return PseudoTable(pseudos)
+    else:
+        return pseudos
 
 
-class StrategyWithInput(object):
-    # TODO: Find a better way to do this. I will likely need to refactor the Strategy object
-    def __init__(self, abinit_input):
-        self.abinit_input = abinit_input
-                                         
-    @property
-    def pseudos(self):
-        return self.abinit_input.pseudos
-
-    def add_extra_abivars(self, abivars):
-        """Add variables (dict) to extra_abivars."""
-        self.abinit_input.set_variables(**abivars)
-
-    def remove_extra_abivars(self, keys):
-        """Remove variables from extra_abivars."""
-        self.abinit_input.remove_variables(keys)
-                                         
-    def make_input(self):
-        return str(self.abinit_input)
+def order_pseudos(pseudos, structure):
+    return select_pseudos(pseudos, structure, ret_table=False)
 
 
 class Strategy(object):
@@ -132,6 +100,14 @@ class Strategy(object):
     def __str__(self):
         return "<%s at %s, accuracy = %s>" % (self.__class__.__name__, id(self), self.accuracy)
 
+    def copy(self):
+        """Shallow copy of self."""
+        return copy.copy(self)
+
+    def deepcopy(self):
+        """Deep copy of self."""
+        return copy.deepcopy(self)
+
     @abc.abstractproperty
     def runlevel(self):
         """String defining the Runlevel. See _runl2optdriver."""
@@ -242,35 +218,10 @@ class Strategy(object):
         for key in keys:
             self.extra_abivars.pop(key)
 
-    #def iocontrol(self):
-    #    "Dictionary with variables controlling the IO"
-    #    ptrwf = self.extra_abivars.get("prtwf")
-    #    
-    #    d = {"prtwf" : prtwf if prtwf is not None else 0
-    #         "prtden": 1,
-    #         "prtgkk": 1 if self.runlevel == "dfpt" else None
-    #        }
-    #    return d
-
-    #def speedup(self):
-    #    "Dictionary with the variables that have a significant impact on the WallTime."
-    #     boxcutmin = self.extra_abivars.get("boxcutmin")
-    #     if boxcutmin is None and self.accuracy == "low": boxcutime = 0.75
-
-    #    "optforces": None if (self.need_forces or want_forces) else 2,
-    #    "optstress": None if (self.need_stress or want_stress) else 0,
-
-    #    d = {"boxcutmin": boxcutmin,
-    #         "optforces": optforces, 
-    #         "optstress": optstress, 
-    #        }
-    #    return d
-
     @abc.abstractmethod
     def make_input(self, *args, **kwargs):
         """Returns an Input instance."""
 
-##########################################################################################
 
 class ScfStrategy(Strategy):
     """
@@ -305,17 +256,17 @@ class ScfStrategy(Strategy):
 
         self.set_accuracy(accuracy)
 
-        self.structure  = structure
-        self.pseudos    = select_pseudos(pseudos, structure)
-        self.ksampling  = ksampling
+        self.structure = structure
+        self.pseudos   = select_pseudos(pseudos, structure)
+        self.ksampling = ksampling
         self.use_symmetries = use_symmetries
 
-        self.electrons  = Electrons(spin_mode = spin_mode,
-                                    smearing  = smearing,
-                                    algorithm = scf_algorithm,
-                                    nband     = None,
-                                    fband     = None,
-                                    charge    = charge,
+        self.electrons  = Electrons(spin_mode=spin_mode,
+                                    smearing=smearing,
+                                    algorithm=scf_algorithm,
+                                    nband=None,
+                                    fband=None,
+                                    charge=charge,
                                    )
 
         self.extra_abivars = extra_abivars
@@ -325,10 +276,11 @@ class ScfStrategy(Strategy):
         return "scf"
 
     def make_input(self):
-        extra = {"optdriver": self.optdriver,
-                 "ecut"     : self.ecut,
-                 "pawecutdg": self.pawecutdg,
-                }
+        extra = dict(
+            optdriver=self.optdriver,
+            ecut=self.ecut,
+            pawecutdg=self.pawecutdg,
+        )
         extra.update(self.tolerance)
         extra.update({"nsym": 1 if not self.use_symmetries else None})
 
@@ -337,7 +289,6 @@ class ScfStrategy(Strategy):
         input = InputWriter(self.structure, self.electrons, self.ksampling, **extra)
         return input.get_string()
 
-##########################################################################################
 
 class NscfStrategy(Strategy):
     """
@@ -373,14 +324,14 @@ class NscfStrategy(Strategy):
         # Electrons used in the GS run.
         scf_electrons = scf_strategy.electrons
 
-        self.electrons  = Electrons(spin_mode = scf_electrons.spin_mode,
-                                    smearing  = scf_electrons.smearing,
-                                    algorithm = nscf_algorithm,
-                                    nband     = nscf_nband,
-                                    fband     = None,
-                                    charge    = scf_electrons.charge,
-                                    comment   = None,
-                                    #occupancies = None,
+        self.electrons = Electrons(spin_mode=scf_electrons.spin_mode,
+                                   smearing=scf_electrons.smearing,
+                                   algorithm=nscf_algorithm,
+                                   nband=nscf_nband,
+                                   fband=None,
+                                   charge=scf_electrons.charge,
+                                   comment=None,
+                                   #occupancies = None,
                                    )
 
         self.extra_abivars = extra_abivars
@@ -393,17 +344,17 @@ class NscfStrategy(Strategy):
         # Initialize the system section from structure.
         scf_strategy = self.scf_strategy
 
-        extra = {"optdriver": self.optdriver,
-                 "ecut"     : self.ecut,
-                 "pawecutdg": self.pawecutdg,
-                }
+        extra = dict(
+            optdriver=self.optdriver,
+            ecut=self.ecut,
+            pawecutdg=self.pawecutdg,
+        )
         extra.update(self.tolerance)
         extra.update(self.extra_abivars)
                                                                                      
         input = InputWriter(scf_strategy.structure, self.electrons, self.ksampling, **extra)
         return input.get_string()
 
-##########################################################################################
 
 class RelaxStrategy(ScfStrategy):
     """Extends ScfStrategy by adding an algorithm for the structural relaxation."""
@@ -453,7 +404,6 @@ class RelaxStrategy(ScfStrategy):
 
         return input_str
 
-##########################################################################################
 
 class ScreeningStrategy(Strategy):
     """Strategy for Screening calculations."""
@@ -492,11 +442,11 @@ class ScreeningStrategy(Strategy):
         if not self.ksampling.is_homogeneous:
             raise ValueError("The k-sampling used for the NSCF run mush be homogeneous")
 
-        self.electrons = Electrons(spin_mode = scf_electrons.spin_mode,
-                                   smearing  = scf_electrons.smearing,
-                                   nband     = scr_nband,
-                                   charge    = scf_electrons.charge,
-                                   comment   = None,
+        self.electrons = Electrons(spin_mode=scf_electrons.spin_mode,
+                                   smearing =scf_electrons.smearing,
+                                   nband=scr_nband,
+                                   charge=scf_electrons.charge,
+                                   comment=None,
                                   )
 
         self.extra_abivars = extra_abivars
@@ -507,18 +457,18 @@ class ScreeningStrategy(Strategy):
 
     def make_input(self):
         # FIXME
-        extra = {"optdriver": self.optdriver,
-                 "ecut"     : self.ecut,
-                 "ecutwfn"  : self.ecut,
-                # "pawecutdg": self.pawecutdg,
-                }
+        extra = dict(
+            optdriver=self.optdriver,
+            ecut=self.ecut,
+            ecutwfn=self.ecut,
+            #pawecutdg=self.pawecutdg,
+        )
         extra.update(self.tolerance)
         extra.update(self.extra_abivars)
                                                                                      
         input = InputWriter(self.scf_strategy.structure, self.electrons, self.ksampling, self.screening, **extra)
         return input.get_string()
 
-##########################################################################################
 
 class SelfEnergyStrategy(Strategy):
     """Strategy for self-energy calculations."""
@@ -572,23 +522,23 @@ class SelfEnergyStrategy(Strategy):
 
     def make_input(self):
         # FIXME
-        extra = {"optdriver": self.optdriver,
-                 "ecut"     : self.ecut,
-                 "ecutwfn"  : self.ecut,
-                # "pawecutdg": self.pawecutdg,
-                }
+        extra = dict(
+            optdriver=self.optdriver,
+            ecut=self.ecut,
+            ecutwfn=self.ecut,
+            # "pawecutdg": self.pawecutdg,
+            )
         extra.update(self.tolerance)
         extra.update(self.extra_abivars)
                                                                                      
         input = InputWriter(self.scf_strategy.structure, self.electrons, self.ksampling, self.sigma, **extra)
         return input.get_string()
 
-##########################################################################################
 
 class MDFBSE_Strategy(Strategy):
     """
-    Strategy for Bethe-Salpeter calculation based on the model dielectric function 
-    and the scissors operator
+    Strategy for Bethe-Salpeter calculation based on the 
+    model dielectric function and the scissors operator
     """
     def __init__(self, scf_strategy, nscf_strategy, exc_ham, **extra_abivars):
         """
@@ -624,30 +574,30 @@ class MDFBSE_Strategy(Strategy):
         if not self.ksampling.is_homogeneous:
             raise ValueError("The k-sampling used for the NSCF run mush be homogeneous")
 
-        self.electrons = Electrons(spin_mode = scf_electrons.spin_mode,
-                                   smearing  = scf_electrons.smearing,
-                                   nband     = exc_ham.nband,
-                                   charge    = scf_electrons.charge,
-                                   comment   = None,
-                                  )
+        self.electrons = Electrons(spin_mode=scf_electrons.spin_mode,
+                                   smearing=scf_electrons.smearing,
+                                   nband=exc_ham.nband,
+                                   charge= scf_electrons.charge,
+                                   comment=None,
+                                    )
     @property
     def runlevel(self):
         return "bse"
 
     def make_input(self):
         # FIXME
-        extra = {"optdriver": self.optdriver,
-                 "ecut"     : self.ecut,
-                 "ecutwfn"  : self.ecut,
-                # "pawecutdg": self.pawecutdg,
-                }
+        extra = dict(
+            optdriver=self.optdriver,
+            ecut=self.ecut,
+            ecutwfn=self.ecut,
+            #pawecutdg=self.pawecutdg,
+        )
         #extra.update(self.tolerance)
         extra.update(self.extra_abivars)
                                                                                      
         input = InputWriter(self.scf_strategy.structure, self.electrons, self.ksampling, self.exc_ham, **extra)
         return input.get_string()
 
-##########################################################################################
 
 
 class InputWriter(object): 
@@ -667,7 +617,7 @@ class InputWriter(object):
                 self.add_extra_abivars(arg)
 
         for (k,v) in kwargs.items():
-            self.add_extra_abivars({k:v})
+            self.add_extra_abivars({k: v})
 
     def __str__(self):
         """String representation (the section of the abinit input file)."""
@@ -717,14 +667,14 @@ class InputWriter(object):
         if isinstance(value, collections.Iterable) and not is_string(value):
             arr = np.array(value)
             if len(arr.shape) in [0,1]: # scalar or vector.
-                token = [key, " ".join([str(i) for i in arr])]
+                token = [key, " ".join(str(i) for i in arr)]
                                                                                    
             else: 
                 # array --> matrix 
                 matrix = np.reshape(arr, (-1, arr.shape[-1])) 
                 lines  = []
                 for (idx, row) in enumerate(matrix):
-                    lines.append(" ".join([str(i) for i in row]))
+                    lines.append(" ".join(str(i) for i in row))
                 token = [key +"\n", "\n".join(lines)]
 
         else:
@@ -765,3 +715,199 @@ class InputWriter(object):
         else:
             return str_delimited(lines, header=None, delimiter=5*" ")
 
+
+class StrategyWithInput(object):
+    # TODO: Find a better way to do this. I will likely need to refactor the Strategy object
+    def __init__(self, abinit_input):
+        self.abinit_input = abinit_input
+                                         
+    @property
+    def pseudos(self):
+        # FIXME: pseudos must be order but I need to define an ABC for the Strategies and Inputs.
+        # Order pseudos
+        pseudos = self.abinit_input.pseudos
+        return order_pseudos(pseudos, self.abinit_input.structure)
+        #print("pseudos", pseudos)
+        #print("ord_pseudos", ord_pseudos)
+        #return ord_pseudos
+        #return self.abinit_input.pseudos
+
+    def add_extra_abivars(self, abivars):
+        """Add variables (dict) to extra_abivars."""
+        self.abinit_input.set_variables(**abivars)
+
+    def remove_extra_abivars(self, keys):
+        """Remove variables from extra_abivars."""
+        self.abinit_input.remove_variables(keys)
+                                         
+    def make_input(self):
+        return str(self.abinit_input)
+
+
+class OpticVar(collections.namedtuple("OpticVar", "name value help")):
+
+    def __str__(self):
+        sval = string(self.value)
+        return (4*" ").join(sval, "!" + self.help)
+
+class OpticInput(object):
+    """
+    abo_1WF7      ! Name of the first d/dk response wavefunction file, produced by abinit
+    abo_1WF8      ! Name of the second d/dk response wavefunction file, produced by abinit
+    abo_1WF9      ! Name of the third d/dk response wavefunction file, produced by abinit
+    abo_WFK       ! Name of the ground-state wavefunction file, produced by abinit
+    0.01          ! Value of the *smearing factor*, in Hartree
+    0.010   1     ! frequency *step* and *maximum* frequency (Ha)
+    0.000         ! *Scissor* shift if needed, in Hartree
+    0.001         ! *Tolerance* on closeness of singularities (in Hartree)
+    3             ! *Number of components* of linear optic tensor to be computed
+    11 33 23      ! Linear *coefficients* to be computed (x=1, y=2, z=3)
+    2             ! Number of components of nonlinear optic tensor to be computed
+    123 222       ! Non-linear coefficients to be computed
+    """
+
+    # variable name --> default value.
+    _VARIABLES = [
+        OpticVar("ddkfile_x", None,  "Name of the first d/dk response wavefunction file"),
+        OpticVar("ddkfile_y", None,  "Name of the second d/dk response wavefunction file"),
+        OpticVar("ddkfile_z", None,  "Name of the third d/dk response wavefunction file"),
+        OpticVar("wfkfile",   None,  "Name of the ground-state wavefunction file"),
+        OpticVar("zcut",      0.01,  "Value of the *smearing factor*, in Hartree"),
+        OpticVar("wmesh",(0.010, 1), "Frequency *step* and *maximum* frequency (Ha)"),
+        OpticVar("scissor", 0.000,   "*Scissor* shift if needed, in Hartree"),
+        OpticVar("sing_tol", 0.001,  "*Tolerance* on closeness of singularities (in Hartree)"), 
+        OpticVar("num_lin_comp", None, "*Number of components* of linear optic tensor to be computed"),
+        OpticVar("lin_comp", None,    "Linear *coefficients* to be computed (x=1, y=2, z=3)"),
+        OpticVar("num_nonlin_comp", None, "Number of components of nonlinear optic tensor to be computed"),
+        OpticVar("nonlin_comp", None, "! Non-linear coefficients to be computed"),
+    ]                                             
+
+    _VARNAMES = [v.name for v in _VARIABLES]
+
+    def __init__(self, **kwargs):
+        # Default values
+        self.vars = collections.OrderedDict((v.name, v.value) for v in _VARIABLES)
+
+        # Update the variables with the values passed by the user
+        for k, v in kwargs:
+            if k not in self.VARNAMES:
+                raise ValueError("varname %s not in %s" % (k, str(self.VARNAMES)))
+            self.vars[k] = v
+
+    def __init__(self, zcut, wstep, wmax, scissor, sing_tol, linear_components, 
+                nonlinear_components=None, ddk_files=None, wfk=None): 
+
+        self.vars = vars = collections.OrderedDict(*self.VAR_NAMES)
+
+        if ddk_files is not None:
+            assert len(ddk_files) == 3
+            assert wfk is not None
+            for dir, ddk in zip(["x", "y", "z"], ddk_files):
+                vars["ddkfile_" + dir] = os.path.abspath(ddk)
+
+        if wfk is not None:
+            vars["wfkfile"] = os.path.abspath(wfk)
+
+        vars["zcut"] = zcut
+        vars["wmesh"] = " ".join(map(str, (wstep, wmax)))
+        vars["sing_tol"] = sing_tol
+
+        vars["num_lin_comp"] = len(linear_components)
+        vars["lin_comp"] = " ".join(str(c) for c in linear_components)
+
+        vars["num_nonlin_comp"] = len(non_linear_components)
+        vars["nonlin_comp"] = " ".join(str(c) for c in nonlinear_components)
+
+    def __init__(self, string):
+        self.string = string
+
+    def __str__(self):
+        return self.string
+
+    def to_string(self):
+        lines = []
+        app = lines.append
+
+        for name in self.VARNAMES:
+            var = self.vars[name]
+            app(str(var))
+
+        return "\n".join(lines)
+
+    def make_input(self):
+        return str(self)
+
+    def add_extra_abivars(self, abivars):
+        """
+        Connection is explicit via the input file
+        since we can pass the paths of the output files 
+        produced by the previous runs.
+        """
+
+
+class AnaddbInput(object):
+
+    def __init__(self, structure=None, ndtset=1, comment=""):
+        """
+        Args:
+            structure:
+                Crystalline structure.
+            ndtset:
+                Number of datasets.
+            comment:
+                Optional string with a comment that will be placed at the beginning of the file.
+        """
+        self._structure = _structure
+        self.ndtset = ndset
+        self.comment = comment
+
+    def __init__(self, string):
+        self.string = string
+
+    @property
+    def structure(self):
+        return self._structure
+
+    def to_string(self):
+        return self.string
+
+    def make_input(self):
+        return self.to_string()
+
+    def add_extra_abivars(self, abivars):
+        """
+        Connection is explicit via the input file
+        since we can pass the paths of the output files 
+        produced by the previous runs.
+        """
+
+    #def set_qpath(self):
+    #    """
+    #    nqpath 7
+    #    qpath
+    #     0.0 0.0 0.0
+    #     1/2 1/2 0.0
+    #     1   1   1
+    #     1/2 1/2 1/2
+    #     1/2 1/2 0.0
+    #     1/2 3/4 1/4
+    #     1/2 1/2 1/2
+    #     """
+
+    #def split_datasets(self):
+    #    """
+    #    Split an input file with multiple datasets into a  list of `ndtset` distinct input files.
+    #    """
+    #    # Propagate subclasses (if any)
+    #    cls = self.__class__ 
+    #    news = []
+
+    #    for i in range(self.ndtset):
+    #        my_vars = self[i+1].allvars
+    #        my_vars.pop("ndtset", None)
+
+    #        new = cls(pseudos=self.pseudos, ndtset=1)
+    #        new.set_variables(**my_vars)
+    #        news.append(new)
+    #
+    #    return news
