@@ -14,6 +14,7 @@ __date__ = "$Sep 23, 2011M$"
 
 import re
 import sys
+import fnmatch
 
 
 def generate_latex_table(results, header=None, caption=None, label=None):
@@ -283,6 +284,61 @@ class StringColorizer(object):
             return string
 
 
+class WildCard(object):
+    """
+    This object provides an easy-to-use interface for
+    filename matching with shell patterns (fnmatch).
+
+    .. example:
+
+    >>> w = WildCard("*.nc|*.pdf")
+    >>> w.filter(["foo.nc", "bar.pdf", "hello.txt"])
+    ['foo.nc', 'bar.pdf']
+
+    >>> w.filter("foo.nc")
+    ['foo.nc']
+    """
+    def __init__(self, wildcard, sep="|"):
+        """
+        Args:
+            wildcard:
+                String of tokens separated by sep.
+                Each token represents a pattern.
+            sep:
+                Separator for shell patterns.
+        """
+        self.pats = ["*"]
+        if wildcard:
+            self.pats = wildcard.split(sep)
+
+    def __str__(self):
+        return "<%s, patterns = %s>" % (self.__class__.__name__, self.pats)
+
+    def filter(self, names): 
+        """
+        Returns a list with the names matching the pattern.
+        """
+        names = list_strings(names)
+
+        fnames = []
+        for f in names:
+            for pat in self.pats:
+                if fnmatch.fnmatch(f, pat):
+                    fnames.append(f)
+
+        return fnames
+
+    def match(self, name):
+        """
+        Returns True if name matches one of the patterns.
+        """
+        for pat in self.pats:
+            if fnmatch.fnmatch(name, pat):
+                return True
+
+        return False
+
+
 if __name__ == "__main__":
     import doctest
     doctest.testmod()
